@@ -160,15 +160,17 @@ subroutine request_dim_grads( fname, dim_len )
   nl=line_number_counter( trim(fname) )
 
   allocate(ctl_cval(nl))
-  allocate(ctl_hval(10,nl))
+  allocate(ctl_hval(10,1))
 
   pdeflag=.false.
   ctl_cval=''
   ctl_hval=''
   tmpc=''
 
+!-- ƬñѿسǼ, ʲ def å
+!-- ƺɸǡܤäƤ뤫Ĵ٤.
+
   call read_file_text( trim(fname), 1, nl, ctl_cval, forma='(a1000)' )
-  call read_file_text( trim(fname), 2, nl-1, ctl_hval(1:2,1:nl-1) )
 
   do i=1,nl
      tmpc=trim(adjustl(ctl_cval(i)))
@@ -190,16 +192,37 @@ subroutine request_dim_grads( fname, dim_len )
      end if
   end do
 
-  if(pdeflag.eqv..true.)then
-     read(ctl_hval(2,cont_flag(1)),*) dim_len(1)
-     read(ctl_hval(2,cont_flag(2)),*) dim_len(2)
-     read(ctl_hval(2,cont_flag(3)),*) dim_len(3)
-     read(ctl_hval(2,cont_flag(4)),*) dim_len(4)
+!-- Υå줿ֹǡºݤ˥ǡɤ߹.
+
+
+  if(pdeflag.eqv..false.)then
+
+     call read_file_text( trim(fname), 2, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(1)-1 )
+     read(ctl_hval(2,1),*) dim_len(1)
+     call read_file_text( trim(fname), 2, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(2)-1 )
+     read(ctl_hval(2,1),*) dim_len(2)
+     call read_file_text( trim(fname), 2, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(3)-1 )
+     read(ctl_hval(2,1),*) dim_len(3)
+     call read_file_text( trim(fname), 2, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(4)-1 )
+     read(ctl_hval(2,1),*) dim_len(4)
+
   else
-     read(ctl_hval(2,cont_flag(5)),*) dim_len(1)
-     read(ctl_hval(3,cont_flag(5)),*) dim_len(2)
-     read(ctl_hval(2,cont_flag(3)),*) dim_len(3)
-     read(ctl_hval(2,cont_flag(4)),*) dim_len(4)
+
+     call read_file_text( trim(fname), 3, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(5)-1 )
+     read(ctl_hval(2,1),*) dim_len(1)
+     read(ctl_hval(3,1),*) dim_len(2)
+     call read_file_text( trim(fname), 2, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(3)-1 )
+     read(ctl_hval(2,1),*) dim_len(3)
+     call read_file_text( trim(fname), 2, 1, ctl_hval(1:2,1),  &
+  &                       skip=cont_flag(4)-1 )
+     read(ctl_hval(2,1),*) dim_len(4)
+
   end if
 
 end subroutine
@@ -213,7 +236,7 @@ subroutine request_axis_grads( fname, axis_name, x, y )
   implicit none
   character(*), intent(in) :: fname  ! GrADS Υȥե
   character(*), intent(in) :: axis_name  ! ɤ߽Ф̾.
-                              ! xdef, ydef, zdef  3 ѥ
+                              ! xdef, ydef, zdef, pdef
   real, intent(inout) :: x(:)  ! κɸ
   real, optional, intent(inout) :: y(:)  ! pdef ξ y κɸ
   character(1000), allocatable, dimension(:) :: ctl_cval
@@ -221,7 +244,7 @@ subroutine request_axis_grads( fname, axis_name, x, y )
   character(100), allocatable, dimension(:,:) :: ctl_hval
   integer :: nl, i, j, nx, ny
   real :: vmin, dv
-  character(1000) :: tmpc
+  character(1000) :: tmpc, tmptmp
   character(100), allocatable, dimension(:) :: dummy
 
   nl=line_number_counter( trim(fname) )
@@ -256,31 +279,77 @@ subroutine request_axis_grads( fname, axis_name, x, y )
      if(tmpc(1:4)==trim(axis_name))then
         if(tmpc(1:4)/='pdef')then
            call read_file_text( trim(fname), 5, 1, ctl_vval(1:5), skip=i-1 )
-           if(trim(ctl_vval(3))=='linear')then
-              read(ctl_vval(4),*) vmin
-              read(ctl_vval(5),*) dv
-              x=(/((vmin+dv*(i-1)),i=1,nx)/)
-           else
-              if(trim(ctl_vval(3))=='levels')then
-                 call read_file_text( trim(fname), nx+3, 1, dummy, skip=i-1 )
-                 do j=4,nx+3
-                    read(dummy(j),*) x(j-3)
-                 end do
+           if(tmpc(1:4)/='tdef')then
+              if(trim(ctl_vval(3))=='linear')then
+                 read(ctl_vval(4),*) vmin
+                 read(ctl_vval(5),*) dv
+                 x=(/((vmin+dv*(i-1)),i=1,nx)/)
               else
-                 write(*,*) "ERROR (request_axis_grads) : ", trim(ctl_vval(3))
-                 write(*,*) "Mode is neither 'levels' nor 'linear'."
-                 write(*,*) "STOP."
-                 stop
+                 if(trim(ctl_vval(3))=='levels')then
+                    call read_file_text( trim(fname), nx+3, 1, dummy, skip=i-1 )
+                    do j=4,nx+3
+                       read(dummy(j),*) x(j-3)
+                    end do
+                 else
+                    write(*,*) "ERROR (request_axis_grads) : ", trim(ctl_vval(3))
+                    write(*,*) "Mode is neither 'levels' nor 'linear'."
+                    write(*,*) "STOP."
+                    stop
+                 end if
+              end if
+           else
+              if(trim(ctl_vval(3))=='linear')then  ! , linear Τб.
+                 tmptmp=trim(adjustl(ctl_vval(5)))
+                 select case (tmptmp(len_trim(tmptmp)-1:len_trim(tmptmp)))
+                 case ('mn')
+                    tmptmp=trim(adjustl(ctl_vval(4)))
+                    read(tmptmp(3:4),*) vmin
+                 case ('hr')
+                    tmptmp=trim(adjustl(ctl_vval(4)))
+                    read(tmptmp(1:2),*) vmin
+                 case ('dy')
+                    tmptmp=trim(adjustl(ctl_vval(4)))
+                    select case (len_trim(tmptmp))
+                    case(12)
+                       read(tmptmp(4:5),*) vmin
+                    case(15)
+                       read(tmptmp(7:8),*) vmin
+                    end select
+                 case ('yr')
+                    tmptmp=trim(adjustl(ctl_vval(4)))
+                    select case (len_trim(tmptmp))
+                    case(12)
+                       read(tmptmp(9:12),*) vmin
+                    case(15)
+                       read(tmptmp(12:15),*) vmin
+                    end select
+                 end select
+
+                 tmptmp=trim(adjustl(ctl_vval(5)))
+                 read(tmptmp(1:len_trim(tmptmp)-2),*) dv
+                 x=(/((vmin+dv*(i-1)),i=1,nx)/)
+!              else
+!                 if(trim(ctl_vval(3))=='levels')then
+!                    call read_file_text( trim(fname), nx+3, 1, dummy, skip=i-1 )
+!                    do j=4,nx+3
+!                       read(dummy(j),*) x(j-3)
+!                    end do
+!                 else
+!                    write(*,*) "ERROR (request_axis_grads) : ", trim(ctl_vval(3))
+!                    write(*,*) "Mode is neither 'levels' nor 'linear'."
+!                    write(*,*) "STOP."
+!                    stop
+!                 end if
               end if
            end if
         else
            call read_file_text( trim(fname), 13, 1, ctl_vval(1:13), skip=i-1 )
            read(ctl_vval(7),*) vmin
            read(ctl_vval(12),*) dv
-           x=(/((vmin+dv*(i-1)),i=1,nx)/)
+           x=(/((dv*(i-int(vmin))),i=1,nx)/)
            read(ctl_vval(8),*) vmin
            read(ctl_vval(13),*) dv
-           y=(/((vmin+dv*(i-1)),i=1,ny)/)
+           y=(/((dv*(i-int(vmin))),i=1,ny)/)
         end if
      end if
   end do
