@@ -76,7 +76,7 @@ subroutine Mean_1d( x, ave, error )  ! 1 ʿͷ׻롼
      if(nt/=0)then
         ave=summ/nt
      else
-        ave=0.0
+        ave=error
      end if
 
   else
@@ -120,7 +120,7 @@ subroutine Mean_2d( x, ave, error )  ! 2 ʿͷ׻롼
      if(nt/=0)then
         ave=summ/nt
      else
-        ave=0.0
+        ave=error
      end if
 
   else
@@ -170,7 +170,7 @@ subroutine Mean_3d( x, ave, error )  ! 3 ʿͷ׻롼
      if(nt/=0)then
         ave=summ/nt
      else
-        ave=0.0
+        ave=error
      end if
 
   else
@@ -852,7 +852,7 @@ end subroutine interpolation_3d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_1d( x, r, u, v, undef, stdopt )
+subroutine auto_interpolation_1d( x, r, u, v, undef, undefr, stdopt )
   ! ɸ x Ƥǡ u 
   ! ɸ r ǡ v ˼ưޤ.
   implicit none
@@ -861,9 +861,12 @@ subroutine auto_interpolation_1d( x, r, u, v, undef, stdopt )
   real, intent(in) :: u(size(x))  ! ǡ
   real, intent(inout) :: v(size(r))  ! ޤǡ
   integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undefr  ! ΰǤ̤.
+        !  undefr ʤ,  undefr 
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, nx, nr, ir
   integer :: defun
+  real :: rdefun
   logical :: stderr
 
   nx=size(x)
@@ -875,6 +878,10 @@ subroutine auto_interpolation_1d( x, r, u, v, undef, stdopt )
      defun=-2147483648
   end if
 
+  if(present(undefr))then
+     rdefun=undefr
+  end if
+
   if(present(stdopt))then
      stderr=stdopt
   else
@@ -885,9 +892,13 @@ subroutine auto_interpolation_1d( x, r, u, v, undef, stdopt )
      call interpo_search_1d( x, r(i), ir, int(defun), stdopt=stderr )
 
      if(ir/=int(defun).and.ir<nx)then
-        call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
+        if(u(ir)/=rdefun.and.u(ir+1)/=rdefun)then
+           call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
+        else
+           v(i)=rdefun
+        end if
      else
-        v(i)=defun
+        v(i)=real(defun)
      end if
   end do
 
@@ -896,7 +907,7 @@ end subroutine auto_interpolation_1d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, stdopt )
+subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, undefr, stdopt )
   ! ɸ x, y Ƥǡ u 
   ! ɸ r, q ǡ v ˼ưޤ.
   implicit none
@@ -907,9 +918,12 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, stdopt )
   real, intent(in) :: u(size(x),size(y))  ! ǡ
   real, intent(inout) :: v(size(r),size(q))  ! ޤǡ
   integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undefr  ! ΰǤ̤.
+        !  undefr ʤ,  undefr 
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, nx, ny, nr, nq, ir, iq
   integer :: defun
+  real :: rdefun
   logical :: stderr
 
   nx=size(x)
@@ -923,6 +937,10 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, stdopt )
      defun=-2147483648
   end if
 
+  if(present(undefr))then
+     rdefun=undefr
+  end if
+
   if(present(stdopt))then
      stderr=stdopt
   else
@@ -935,10 +953,15 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, stdopt )
   &                             stdopt=stderr )
 
         if(ir/=int(defun).and.iq/=int(defun).and.ir<nx.and.iq<ny)then
-           call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
-  &                               u(ir:ir+1,iq:iq+1), (/r(i), q(j)/), v(i,j) )
+           if(u(ir,iq)/=rdefun.and.u(ir+1,iq)/=rdefun.and.  &
+  &           v(ir,iq+1)/=rdefun.and.v(ir+1,iq+1)/=rdefun)then
+              call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
+  &                              u(ir:ir+1,iq:iq+1), (/r(i), q(j)/), v(i,j) )
+           else
+              v(i,j)=rdefun
+           end if
         else
-           v(i,j)=defun
+           v(i,j)=real(defun)
         end if
      end do
   end do
@@ -948,7 +971,7 @@ end subroutine auto_interpolation_2d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, stdopt )
+subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, undefr, stdopt )
   ! ɸ x, y, z Ƥǡ u 
   ! ɸ r, q, p ǡ v ˼ưޤ.
   implicit none
@@ -961,9 +984,12 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, stdopt )
   real, intent(in) :: u(size(x),size(y),size(z))  ! ǡ
   real, intent(inout) :: v(size(r),size(q),size(p))  ! ޤǡ
   integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undefr  ! ΰǤ̤.
+        !  undefr ʤ,  undefr 
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, k, nx, ny, nz, nr, nq, np, ir, iq, ip
   integer :: defun
+  real :: rdefun
   logical :: stderr
 
   nx=size(x)
@@ -979,6 +1005,10 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, stdopt )
      defun=-2147483648
   end if
 
+  if(present(undefr))then
+     rdefun=undefr
+  end if
+
   if(present(stdopt))then
      stderr=stdopt
   else
@@ -993,11 +1023,18 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, stdopt )
 
            if(ir/=int(defun).and.iq/=int(defun).and.ip/=int(defun).and.  &
   &           ir<nx.and.iq<ny.and.ip<nz)then
-              call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
-  &                               u(ir:ir+1,iq:iq+1,ip:ip+1),  &
-  &                               (/r(i), q(j), p(k)/), v(i,j,k) )
+              if(u(ir,iq,ip)/=rdefun.and.u(ir+1,iq,ip)/=rdefun.and.  &
+  &              u(ir,iq+1,ip)/=rdefun.and.u(ir+1,iq+1,ip)/=rdefun.and.  &
+  &              u(ir,iq,ip+1)/=rdefun.and.u(ir+1,iq,ip+1)/=rdefun.and.  &
+  &              u(ir,iq+1,ip+1)/=rdefun.and.u(ir+1,iq+1,ip+1)/=rdefun)then
+                 call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
+  &                                  u(ir:ir+1,iq:iq+1,ip:ip+1),  &
+  &                                  (/r(i), q(j), p(k)/), v(i,j,k) )
+              else
+                 v(i,j,k)=rdefun
+              end if
            else
-              v(i,j,k)=defun
+              v(i,j,k)=real(defun)
            end if
         end do
      end do
@@ -1520,6 +1557,355 @@ end subroutine Cor_Coe_3d
 !---------------------------------
 !---------------------------------
 
+subroutine smooth_1d( x, n, y, method, weight, error )
+! 1 ǡˤĤ, ࡼ󥰤롼
+! , error ץϵǽƤʤ.
+  implicit none
+  real, intent(in) :: x(:)  ! ࡼ󥰤ǡ
+  integer, intent(in) :: n  ! ࡼαƶʻҿ(濴ޤ᤿)
+  real, intent(inout) :: y(size(x))  ! ࡼ󥰤줿ǡ
+  character(3), intent(in) :: method  ! ࡼ󥰤ˡ
+                            ! "SMP" = ñʿ, "OPT" = ץŤ
+                            ! "MAX" = , "MIN" = Ǿ
+  real, intent(in), optional :: weight(n)  ! method  "OPT" ξ
+                            ! weight(1) ü, weight(n) üȤƽŤ
+  real, intent(in), optional :: error  ! ̤
+  integer :: ix, j, mx, half, val
+  real :: div_fact
+  real :: wg(n)
+
+  y=0.0
+  mx=size(x)
+  half=(n-1)/2
+
+  select case (method(1:3))
+  case ("SMP")
+     do j=1,n
+        wg(j)=1.0
+     end do
+
+  case ("OPT")
+     if(present(weight))then
+        do j=1,n
+           wg(j)=weight(j)
+        end do
+     end if
+  end select
+
+  if(method(1:3)/="MIN".and.method(1:3)/="MAX")then
+!-- determining dividing factor
+
+     div_fact=0.0
+
+     do j=1,n
+        if(wg(j)<0.0)then
+           div_fact=1.0
+           exit
+        else
+           div_fact=div_fact+wg(j)
+        end if
+     end do
+
+!-- avoiding zero dividing
+
+     if(div_fact==0.0)then
+        div_fact=1.0
+     end if
+
+     do ix=half+1,mx-half
+        do j=1,n
+           y(ix)=y(ix)+x(ix-half-1+j)*wg(j)
+        end do
+        y(ix)=y(ix)/div_fact
+     end do
+
+  else
+
+     select case (method(1:3))
+     case ("MAX")
+        do ix=half+1,mx-half
+           val=x(ix-half)
+           do j=2,n
+              if(val<x(ix-half-1+j))then
+                 val=x(ix-half-1+j)
+              end if
+           end do
+           y(ix-half)=val
+        end do
+
+     case ("MIN")
+        do ix=half+1,mx-half
+           val=x(ix-half)
+           do j=2,n
+              if(val>x(ix-half-1+j))then
+                 val=x(ix-half-1+j)
+              end if
+           end do
+           y(ix-half)=val
+        end do
+
+     end select
+
+  end if
+
+end subroutine smooth_1d
+
+!---------------------------------
+!---------------------------------
+
+subroutine smooth_2d( x, n, y, method, weight, error )
+! 2 ǡˤĤ, ࡼ󥰤롼
+! , error ץϵǽƤʤ.
+  implicit none
+  real, intent(in) :: x(:,:)  ! ࡼ󥰤ǡ
+  integer, intent(in) :: n  ! ࡼαƶʻҿ(濴ޤ᤿)
+  real, intent(inout) :: y(size(x,1),size(x,2))  ! ࡼ󥰤줿ǡ
+  character(3), intent(in) :: method  ! ࡼ󥰤ˡ
+                            ! "SMP" = ñʿ, "OPT" = ץŤ
+                            ! "MAX" = , "MIN" = Ǿ
+  real, intent(in), optional :: weight(n,n)  ! method  "OPT" ξ
+                          ! weight(1,1) ü, weight(n,n) üȤƽŤ
+  real, intent(in), optional :: error  ! ̤
+  integer :: ix, iy, j, k, mx, my, half, val
+  real :: div_fact
+  real :: wg(n,n)
+
+  y=0.0
+  mx=size(x,1)
+  my=size(x,2)
+  half=(n-1)/2
+
+  select case (method(1:3))
+  case ("SMP")
+     do k=1,n
+        do j=1,n
+           wg(j,k)=1.0
+        end do
+     end do
+
+  case ("OPT")
+     if(present(weight))then
+        do k=1,n
+           do j=1,n
+              wg(j,k)=weight(j,k)
+           end do
+        end do
+     end if
+  end select
+
+  if(method(1:3)/="MIN".and.method(1:3)/="MAX")then
+!-- determining dividing factor
+
+     div_fact=0.0
+
+     do k=1,n
+        do j=1,n
+           if(wg(j,k)<0.0)then
+              div_fact=1.0
+              exit
+           else
+              div_fact=div_fact+wg(j,k)
+           end if
+        end do
+     end do
+
+!-- avoiding zero dividing
+
+     if(div_fact==0.0)then
+        div_fact=1.0
+     end if
+
+     do iy=half+1,my-half
+        do ix=half+1,mx-half
+           do k=1,n
+              do j=1,n
+                 y(ix,iy)=y(ix,iy)+x(ix-half-1+j,iy-half-1+k)*wg(j,k)
+              end do
+           end do
+
+           y(ix,iy)=y(ix,iy)/div_fact
+        end do
+     end do
+
+  else
+
+     select case (method(1:3))
+     case ("MAX")
+        do iy=half+1,my-half
+           do ix=half+1,mx-half
+              val=x(ix-half,iy-half)
+              do k=2,n
+                 do j=2,n
+                    if(val<x(ix-half-1+j,iy-half-1+k))then
+                       val=x(ix-half-1+j,iy-half-1+k)
+                    end if
+                 end do
+              end do
+              y(ix-half,iy-half)=val
+           end do
+        end do
+
+     case ("MIN")
+        do iy=half+1,my-half
+           do ix=half+1,mx-half
+              val=x(ix-half,iy-half)
+              do k=2,n
+                 do j=2,n
+                    if(val>x(ix-half-1+j,iy-half-1+k))then
+                       val=x(ix-half-1+j,iy-half-1+k)
+                    end if
+                 end do
+              end do
+              y(ix-half,iy-half)=val
+           end do
+        end do
+
+     end select
+
+  end if
+
+end subroutine smooth_2d
+
+!---------------------------------
+!---------------------------------
+
+subroutine smooth_3d( x, n, y, method, weight, error )
+! 3 ǡˤĤ, ࡼ󥰤롼
+! , error ץϵǽƤʤ.
+  implicit none
+  real, intent(in) :: x(:,:,:)  ! ࡼ󥰤ǡ
+  integer, intent(in) :: n  ! ࡼαƶʻҿ(濴ޤ᤿)
+  real, intent(inout) :: y(size(x,1),size(x,2),size(x,3))
+                            ! ࡼ󥰤줿ǡ
+  character(3), intent(in) :: method  ! ࡼ󥰤ˡ
+                            ! "SMP" = ñʿ, "OPT" = ץŤ
+                            ! "MAX" = , "MIN" = Ǿ
+  real, intent(in), optional :: weight(n,n,n)  ! method  "OPT" ξ
+  real, intent(in), optional :: error  ! ̤
+  integer :: ix, iy, iz, j, k, l, mx, my, mz, half, val
+  real :: div_fact
+  real :: wg(n,n,n)
+
+  y=0.0
+  mx=size(x,1)
+  my=size(x,2)
+  mz=size(x,3)
+  half=(n-1)/2
+
+  select case (method(1:3))
+  case ("SMP")
+     do l=1,n
+        do k=1,n
+           do j=1,n
+              wg(j,k,l)=1.0
+           end do
+        end do
+     end do
+
+  case ("OPT")
+     if(present(weight))then
+        do l=1,n
+           do k=1,n
+              do j=1,n
+                 wg(j,k,l)=weight(j,k,l)
+              end do
+           end do
+        end do
+     end if
+  end select
+
+  if(method(1:3)/="MIN".and.method(1:3)/="MAX")then
+!-- determining dividing factor
+
+     div_fact=0.0
+
+     do l=1,n
+        do k=1,n
+           do j=1,n
+              if(wg(j,k,l)<0.0)then
+                 div_fact=1.0
+                 exit
+              else
+                 div_fact=div_fact+wg(j,k,l)
+              end if
+           end do
+        end do
+     end do
+
+!-- avoiding zero dividing
+
+     if(div_fact==0.0)then
+        div_fact=1.0
+     end if
+
+     do iz=half+1,mz-half
+        do iy=half+1,my-half
+           do ix=half+1,mx-half
+              do l=1,n
+                 do k=1,n
+                    do j=1,n
+                       y(ix,iy,iz)=y(ix,iy,iz)  &
+  &                                +x(ix-half-1+j,iy-half-1+k,iz-half-1+l)  &
+  &                                *wg(j,k,l)
+                    end do
+                 end do
+              end do
+
+              y(ix,iy,iz)=y(ix,iy,iz)/div_fact
+           end do
+        end do
+     end do
+
+  else
+
+     select case (method(1:3))
+     case ("MAX")
+        do iz=half+1,mz-half
+           do iy=half+1,my-half
+              do ix=half+1,mx-half
+                 val=x(ix-half,iy-half,iz-half)
+                 do l=2,n
+                    do k=2,n
+                       do j=2,n
+                          if(val<x(ix-half-1+j,iy-half-1+k,iz-half-1+l))then
+                             val=x(ix-half-1+j,iy-half-1+k,iz-half-1+l)
+                          end if
+                       end do
+                    end do
+                 end do
+                 y(ix-half,iy-half,iz-half)=val
+              end do
+           end do
+        end do
+
+     case ("MIN")
+        do iz=half+1,mz-half
+           do iy=half+1,my-half
+              do ix=half+1,mx-half
+                 val=x(ix-half,iy-half,iz-half)
+                 do l=2,n
+                    do k=2,n
+                       do j=2,n
+                          if(val>x(ix-half-1+j,iy-half-1+k,iz-half-1+l))then
+                             val=x(ix-half-1+j,iy-half-1+k,iz-half-1+l)
+                          end if
+                       end do
+                    end do
+                 end do
+                 y(ix-half,iy-half,iz-half)=val
+              end do
+           end do
+        end do
+
+     end select
+
+  end if
+
+end subroutine smooth_3d
+
+!---------------------------------
+!---------------------------------
+
 subroutine Move_ave( x, n, y, error )
 ! ưʿѤ׻롼
   implicit none
