@@ -8,6 +8,7 @@ use Thermo_Function
 use Thermo_Routine
 use Thermo_Advanced_Function
 use Derivation
+use Algebra
 
 contains
 
@@ -394,7 +395,7 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
      call curl_3d( x, y, z, u, v, w, tmp4, tmp5, tmp6, undeff,  & 
   &               scalex, scaley, scalez )
 !  omega  grad pt Ѥ׻
-     call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7, undeff )
+     call dot_prod_3d( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7, undeff )
   else
 !  ̤ζָۤ׻.
      call grad_3d( x, y, z, pt, tmp1, tmp2, tmp3,  &
@@ -403,7 +404,7 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
      call curl_3d( x, y, z, u, v, w, tmp4, tmp5, tmp6,  & 
   &               hx=scalex, hy=scaley, hz=scalez )
 !  omega  grad pt Ѥ׻
-     call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7 )
+     call dot_prod_3d( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7 )
   end if
 
   if(present(undeff))then
