@@ -4,7 +4,11 @@
 
 module typhoon_analy  ! ټ¸ѥڥϥ⥸塼
 
-  private :: search_region_1d  ! ʿѤǽȾ¤ܺ.
+  use Statistics
+
+  implicit none
+
+!  private :: search_region_1d  ! ʿѤǽȾ¤ܺ.
 
 contains
 
@@ -111,20 +115,18 @@ subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v, undef,  &
         tmpz(2,1)=u(ip(i,j,1)+1,ip(i,j,2))
         tmpz(1,2)=u(ip(i,j,1),ip(i,j,2)+1)
         tmpz(2,2)=u(ip(i,j,1)+1,ip(i,j,2)+1)
+        inter(1)=point(i,j,1)
+        inter(2)=point(i,j,2)
 
         if(present(undefg))then
            ucf=undef_checker_2d( tmpz, undefg )
            if(ucf.eqv..false.)then
-              inter(1)=point(i,j,1)
-              inter(2)=point(i,j,2)
               call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
            else
               work(i,j)=undefg
               undefgc_check(i)=.false.
            end if
         else
-           inter(1)=point(i,j,1)
-           inter(2)=point(i,j,2)
            call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
         end if
      end do
@@ -264,20 +266,18 @@ subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v, undef,  &
         tmpz(2,1)=u(ip(i,j,1)+1,ip(i,j,2))
         tmpz(1,2)=u(ip(i,j,1),ip(i,j,2)+1)
         tmpz(2,2)=u(ip(i,j,1)+1,ip(i,j,2)+1)
+        inter(1)=point(i,j,1)
+        inter(2)=point(i,j,2)
 
         if(present(undefg))then
            ucf=undef_checker_2d( tmpz, undefg )
            if(ucf.eqv..false.)then
-              inter(1)=point(i,j,1)
-              inter(2)=point(i,j,2)
               call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
            else
               work(i,j)=undefg
               undefgc_check(i)=.false.
            end if
         else
-           inter(1)=point(i,j,1)
-           inter(2)=point(i,j,2)
            call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
         end if
      end do
@@ -443,13 +443,16 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v,  &
   ! (3) ޤ 4 Ф, ǤΥ顼ͤ 4 Υ顼
   !     ,  interpolation_2d Ƿ׻.
   !     Ʊ롼, 2 ĤΥ٥ȥʬʿͤΤ,
-  !     Ѥ vec_prod ˤä濴ΰ֥٥ȥȤγѤ׻
+  !     Ѥ vec_prod_2d ˤä濴ΰ֥٥ȥȤγѤ׻
   !     . Ʊ롼濴εΥǳä v ʬΤ.
   ! (4) nr x nt Ĥʬͤޤä, nt ʿѷ׻ mean_1d .
   ! ʾǳ nr ˤĤʿͤ.
   ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
   ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
   ! βս򻲾Ȥ줿.
+  ! ʤ, ܥ롼ϲΤ, β tangent_mean_scal 롼
+  ! ꤲƤ. ܥ롼ǹԤΤ, charc ˹碌,
+  ! ƥǥȺɸˤ濴Ф, ư׻ΤߤǤ.
   use algebra
   use Derivation
   use max_min
@@ -471,207 +474,88 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v,  &
   character(3), intent(in), optional :: undefgc  ! undefg ν
                 ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
                 ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
-  integer :: i, j, nx, ny, nr, nrr, nt
-  real :: work1(size(r),size(theta),1), work2(size(r),size(theta),1), work3(size(r),size(theta),1)
-  real :: posx(size(r),size(theta),1), posy(size(r),size(theta),1), posz(size(r),size(theta),1)
-  real :: vecx(size(r),size(theta),1), vecy(size(r),size(theta),1), vecz(size(r),size(theta),1)
-  real :: abpos(size(r),size(theta))
-  real :: point(size(r),size(theta),2)
-  integer :: ip(size(r),size(theta),2)
-  real :: tmpx(2), tmpy(2), tmpz(2,2), inter(2)
-  character(1) :: undefgcflag
-  logical, dimension(size(r)) :: undefgc_check
-  logical :: ucf
+  integer :: i, j, nx, ny, z_count
+  real, dimension(size(x),size(y)) :: posx, posy, abpos, vecz
 
   nx=size(x)
   ny=size(y)
-  nrr=size(r)
-  nt=size(theta)
+  z_count=0
 
-  undefgc_check(:)=.true.
+!-- ޤ, charc ˹碌, ʿ٥ȥư¡Ѵ.
+!-- Ѵ, ͤϥ顼ͤȤɾΤ,
+!-- tangent_mean_scal ꤲ.
 
-  if(present(undefg))then
-     if(present(undefgc))then
-        undefgcflag=undefgc(1:1)
-     else
-        undefgcflag="i"
-     end if
-  end if
+!-- 濴ФƥǥȺɸοʿ֥٥ȥ׻.
 
-!-- ΰ򥯥ꥢƤ뤫ǧ ---
-
-  call search_region_1d( (/x(1),x(nx)/), (/y(1),y(ny)/), (/xc,yc/), r, nr )
-
-!--  v  undef ͤƤ.
-  do i=1,nrr
-     if(present(undef))then
-        v(i)=undef
-     else
-        v(i)=0.0
-     end if
-  end do
-
-!-- (1) ---
-  do j=1,nt
-     do i=1,nr
-        call rt_2_xy( r(i), theta(j), point(i,j,1), point(i,j,2) )
-        point(i,j,1)=xc+point(i,j,1)
-        point(i,j,2)=yc+point(i,j,2)
+  do j=1,ny
+     do i=1,nx
+        posx(i,j)=x(i)-xc
+        posy(i,j)=y(j)-yc
      end do
   end do
 
-!-- (2) ---
-  do j=1,nt
-     do i=1,nr
-        call interpo_search_2d( x, y, point(i,j,1), point(i,j,2),  &
-      &                         ip(i,j,1), ip(i,j,2) )
-     end do
-  end do
-
-
-!-- (3) ---
-!-- 1. ٥ȥ 2 ʬˤĤޤͤ˳Ǽ ---
-  if(present(undefg))then
-     do j=1,nt
-        do i=1,nr
-           tmpx(1)=x(ip(i,j,1))
-           tmpx(2)=x(ip(i,j,1)+1)
-           tmpy(1)=y(ip(i,j,2))
-           tmpy(2)=y(ip(i,j,2)+1)
-           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
-           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
-           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
-           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
-           ucf=undef_checker_2d( tmpz, undefg )
-           if(ucf.eqv..false.)then
-              inter(1)=point(i,j,1)
-              inter(2)=point(i,j,2)
-
-              call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
-
-              tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
-              tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
-              tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
-              tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
-              ucf=undef_checker_2d( tmpz, undefg )
-              if(ucf.eqv..false.)then
-                 call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
-              else
-                 work1(i,j,1)=undefg
-                 work2(i,j,1)=undefg
-              end if
-           else  ! λ undefg äƤ, work2 Ȥʤ.
-              work1(i,j,1)=undefg
-              work2(i,j,1)=undefg
-           end if
-        end do
-     end do
-  else
-     do j=1,nt
-        do i=1,nr
-           tmpx(1)=x(ip(i,j,1))
-           tmpx(2)=x(ip(i,j,1)+1)
-           tmpy(1)=y(ip(i,j,2))
-           tmpy(2)=y(ip(i,j,2)+1)
-           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
-           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
-           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
-           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
-           inter(1)=point(i,j,1)
-           inter(2)=point(i,j,2)
-           call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
-           tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
-           tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
-           tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
-           tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
-           call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
-        end do
-     end do
-  end if
-
-!-- 2. ޤȤä, Ǥ x,y ɸ(ֹ, ix,iy ˳Ǽ)
-!--    ΰ֥٥ȥȤγѤ׻
-  do j=1,nt
-     do i=1,nr
-        work3(i,j,1)=0.0
-        posx(i,j,1)=point(i,j,1)-xc
-        posy(i,j,1)=point(i,j,2)-yc
-        posz(i,j,1)=0.0
-     end do
-  end do
-
-  select case (charc)
+  select case (trim(charc))
   case ('vector')
-     call vec_prod( posx, posy, posz, work1, work2, work3, vecx, vecy, vecz )
+     if(present(undefg))then
+        call vec_prod_2d( posx, posy, u1, u2, vecz, undeff=undefg )
+     else
+        call vec_prod_2d( posx, posy, u1, u2, vecz )
+     end if
   case ('scalar')
-     call dot_prod( posx, posy, posz, work1, work2, work3, vecz )
+     if(present(undefg))then
+        call dot_prod_2d( posx, posy, u1, u2, vecz, undeff=undefg )
+     else
+        call dot_prod_2d( posx, posy, u1, u2, vecz )
+     end if
   case default
      write(*,*) "error : bad character. select 'vector', or 'scalar'."
      stop
   end select
 
-!-- , work1  undefg äƤ뤫å.
-!-- , work1  undefg äƤ, work2 ˤ undefg 
-!-- äƤΤ, work1 åФ褤.
+  call abst_2d( posx, posy, abpos )
 
   if(present(undefg))then
-     do j=1,nt
-        do i=1,nr
-           if(work1(i,j,1)==undefg)then
-              vecz(i,j,1)=undefg
-           end if
-        end do
-     end do
-  end if
-
-!-- 3. ٥ȥγʬΤ, z ʬˤĤ (2 ʿ̥٥ȥƱΤγ)
-!--    ֥٥ȥͤǳ. -> ®ʬޤͤ.
-  call abst( posx, posy, posz, abpos )
-
-  if(present(undefg))then
-     do j=1,nt
-        do i=2,nr   
-           if(vecz(i,j,1)/=undefg)then
-              vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+     do j=1,ny
+        do i=1,nx   
+           if(vecz(i,j)/=undefg.and.abpos(i,j)/=0.0)then
+              vecz(i,j)=vecz(i,j)/abpos(i,j)
+           else if(abpos(i,j)==0.0)then
+              z_count=z_count+1
+              vecz(i,j)=0.0
            end if
         end do
      end do
   else
-     do j=1,nt
-        do i=2,nr   
-           vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
-        end do
-     end do
-  end if
-
-
-!-- (4) ---
-  if(present(undefg))then
-     if(undefgcflag(1:1)=='i')then
-        do i=2,nr
-           call Mean_1d( vecz(i,:,1), v(i), error=undefg )
-        end do
-     else
-        do i=2,nr
-           ucf=undef_checker_1d( vecz(i,:,1), undefg )
-           if(ucf.eqv..false.)then
-              call Mean_1d( vecz(i,:,1), v(i) )
+     do j=1,ny
+        do i=1,nx
+           if(abpos(i,j)/=0.0)then
+              vecz(i,j)=vecz(i,j)/abpos(i,j)
            else
-              v(i)=undefg
+              z_count=z_count+1
+              vecz(i,j)=0.0
            end if
         end do
-     end if
-  else
-     do i=2,nr
-        call Mean_1d( vecz(i,:,1), v(i) )
      end do
   end if
 
-  v(1)=0.0
+  if(present(undef))then
+     if(present(undefg))then
+        call tangent_mean_scal( x, y, xc, yc, vecz, r, theta, v,  &
+  &                             undef=undef, undefg=undefg,  &
+  &                             undefgc=trim(undefgc) )
+     else
+        call tangent_mean_scal( x, y, xc, yc, vecz, r, theta, v,  &
+  &                             undef=undef )
+     end if
+  else if(present(undefg))then
+     call tangent_mean_scal( x, y, xc, yc, vecz, r, theta, v,  &
+  &                          undefg=undefg, undefgc=trim(undefgc) )
+  else
+     call tangent_mean_scal( x, y, xc, yc, vecz, r, theta, v )
+  end if
 
 end subroutine tangent_mean_vec
 
-
 !--------------------------------------------------------------
 !--------------------------------------------------------------
 
@@ -688,13 +572,16 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v,  &
   ! (3) ޤ 4 Ф, ǤΥ顼ͤ 4 Υ顼
   !     ,  interpolation_2d Ƿ׻.
   !     Ʊ롼, 2 ĤΥ٥ȥʬʿͤΤ,
-  !     Ѥ vec_prod ˤä濴ΰ֥٥ȥȤγѤ׻
+  !     Ѥ vec_prod_2d ˤä濴ΰ֥٥ȥȤγѤ׻
   !     . Ʊ롼濴εΥǳä v ʬΤ.
   ! (4) nr x nt Ĥʬͤޤä, nt ʿѷ׻ mean_1d .
   ! ʾǳ nr ˤĤʿͤ.
   ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
   ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
   ! βս򻲾Ȥ줿.
+  ! ʤ, ܥ롼ϲΤ, β tangent_mean_scal 롼
+  ! ꤲƤ. ܥ롼ǹԤΤ, charc ˹碌,
+  ! ƥǥȺɸˤ濴Ф, ư׻ΤߤǤ.
   use algebra
   use Derivation
   use max_min
@@ -716,205 +603,85 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v,  &
   character(3), intent(in), optional :: undefgc  ! undefg ν
                 ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
                 ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
-  integer :: i, j, nx, ny, nr, nrr, nt
-  real :: work1(size(r),size(theta),1), work2(size(r),size(theta),1), work3(size(r),size(theta),1)
-  real :: posx(size(r),size(theta),1), posy(size(r),size(theta),1), posz(size(r),size(theta),1)
-  real :: vecx(size(r),size(theta),1), vecy(size(r),size(theta),1), vecz(size(r),size(theta),1)
-  real :: abpos(size(r),size(theta))
-  real :: point(size(r),size(theta),2)
-  integer :: ip(size(r),size(theta),2)
-  real :: tmpx(2), tmpy(2), tmpz(2,2), inter(2)
-  character(1) :: undefgcflag
-  logical, dimension(size(r)) :: undefgc_check
-  logical :: ucf
+  integer :: i, j, nx, ny, z_count
+  real, dimension(size(x),size(y)) :: posx, posy, abpos, vecz
 
   nx=size(x)
   ny=size(y)
-  nrr=size(r)
-  nt=size(theta)
+  z_count=0
 
-  undefgc_check(:)=.true.
+!-- ޤ, charc ˹碌, ʿ٥ȥư¡Ѵ.
+!-- Ѵ, ͤϥ顼ͤȤɾΤ,
+!-- tangent_mean_scal ꤲ.
 
-  if(present(undefg))then
-     if(present(undefgc))then
-        undefgcflag=undefgc(1:1)
-     else
-        undefgcflag="i"
-     end if
-  end if
+!-- 濴ФƥǥȺɸοʿ֥٥ȥ׻.
 
-!-- ΰ򥯥ꥢƤ뤫ǧ ---
-
-  call search_region_1d( (/x(1),x(nx)/), (/y(1),y(ny)/), (/xc,yc/), r, nr )
-
-!--  v  undef ͤƤ.
-  do j=1,nt
-     do i=1,nrr
-        if(present(undef))then
-           v(i,j)=undef
-        else
-           v(i,j)=0.0
-        end if
+  do j=1,ny
+     do i=1,nx
+        posx(i,j)=x(i)-xc
+        posy(i,j)=y(j)-yc
      end do
   end do
 
-!-- (1) ---
-  do j=1,nt
-     do i=1,nr
-        call rt_2_xy( r(i), theta(j), point(i,j,1), point(i,j,2) )
-        point(i,j,1)=xc+point(i,j,1)
-        point(i,j,2)=yc+point(i,j,2)
-     end do
-  end do
-
-!-- (2) ---
-  do j=1,nt
-     do i=1,nr
-        call interpo_search_2d( x, y, point(i,j,1), point(i,j,2),  &
-      &                         ip(i,j,1), ip(i,j,2) )
-     end do
-  end do
-
-
-!-- (3) ---
-!-- 1. ٥ȥ 2 ʬˤĤޤͤ˳Ǽ ---
-  if(present(undefg))then
-     do j=1,nt
-        do i=1,nr
-           tmpx(1)=x(ip(i,j,1))
-           tmpx(2)=x(ip(i,j,1)+1)
-           tmpy(1)=y(ip(i,j,2))
-           tmpy(2)=y(ip(i,j,2)+1)
-           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
-           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
-           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
-           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
-           ucf=undef_checker_2d( tmpz, undefg )
-           if(ucf.eqv..false.)then
-              inter(1)=point(i,j,1)
-              inter(2)=point(i,j,2)
-
-              call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
-
-              tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
-              tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
-              tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
-              tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
-              ucf=undef_checker_2d( tmpz, undefg )
-              if(ucf.eqv..false.)then
-                 call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
-              else
-                 work1(i,j,1)=undefg
-                 work2(i,j,1)=undefg
-              end if
-           else  ! λ undefg äƤ, work2 Ȥʤ.
-              work1(i,j,1)=undefg
-              work2(i,j,1)=undefg
-           end if
-        end do
-     end do
-  else
-     do j=1,nt
-        do i=1,nr
-           tmpx(1)=x(ip(i,j,1))
-           tmpx(2)=x(ip(i,j,1)+1)
-           tmpy(1)=y(ip(i,j,2))
-           tmpy(2)=y(ip(i,j,2)+1)
-           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
-           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
-           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
-           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
-           inter(1)=point(i,j,1)
-           inter(2)=point(i,j,2)
-           call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
-           tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
-           tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
-           tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
-           tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
-           call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
-        end do
-     end do
-  end if
-
-!-- 2. ޤȤä, Ǥ x,y ɸ(ֹ, ix,iy ˳Ǽ)
-!--    ΰ֥٥ȥȤγѤ׻
-  do j=1,nt
-     do i=1,nr
-        work3(i,j,1)=0.0
-        posx(i,j,1)=point(i,j,1)-xc
-        posy(i,j,1)=point(i,j,2)-yc
-        posz(i,j,1)=0.0
-     end do
-  end do
-
-  select case (charc)
+  select case (trim(charc))
   case ('vector')
-     call vec_prod( posx, posy, posz, work1, work2, work3, vecx, vecy, vecz )
+     if(present(undefg))then
+        call vec_prod_2d( posx, posy, u1, u2, vecz, undeff=undefg )
+     else
+        call vec_prod_2d( posx, posy, u1, u2, vecz )
+     end if
   case ('scalar')
-     call dot_prod( posx, posy, posz, work1, work2, work3, vecz )
+     if(present(undefg))then
+        call dot_prod_2d( posx, posy, u1, u2, vecz, undeff=undefg )
+     else
+        call dot_prod_2d( posx, posy, u1, u2, vecz )
+     end if
   case default
      write(*,*) "error : bad character. select 'vector', or 'scalar'."
      stop
   end select
 
-!-- , work1  undefg äƤ뤫å.
-!-- , work1  undefg äƤ, work2 ˤ undefg 
-!-- äƤΤ, work1 åФ褤.
+  call abst_2d( posx, posy, abpos )
 
   if(present(undefg))then
-     do j=1,nt
-        do i=1,nr
-           if(work1(i,j,1)==undefg)then
-              vecz(i,j,1)=undefg
-           end if
-        end do
-     end do
-  end if
-
-!-- 3. ٥ȥγʬΤ, z ʬˤĤ (2 ʿ̥٥ȥƱΤγ)
-!--    ֥٥ȥͤǳ. -> ®ʬޤͤ.
-  call abst( posx, posy, posz, abpos )
-
-  if(present(undefg))then
-     do j=1,nt
-        do i=2,nr
-           if(vecz(i,j,1)/=undefg)then
-              vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+     do j=1,ny
+        do i=1,nx   
+           if(vecz(i,j)/=undefg.and.abpos(i,j)/=0.0)then
+              vecz(i,j)=vecz(i,j)/abpos(i,j)
+           else if(abpos(i,j)==0.0)then
+              z_count=z_count+1
+              vecz(i,j)=0.0
            end if
         end do
      end do
   else
-     do j=1,nt
-        do i=2,nr
-           vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
-        end do
-     end do
-  end if
-
-
-!-- (4) ---
-  if(present(undefg))then
-     if(undefgcflag(1:1)=='i')then
-        do i=2,nr
-           call Anomaly_1d( vecz(i,:,1), v(i,:), error=undefg )
-        end do
-     else
-        do i=2,nr
-           ucf=undef_checker_1d( vecz(i,:,1), undefg )
-           if(ucf.eqv..false.)then
-              call Anomaly_1d( vecz(i,:,1), v(i,:) )
+     do j=1,ny
+        do i=1,nx
+           if(abpos(i,j)/=0.0)then
+              vecz(i,j)=vecz(i,j)/abpos(i,j)
            else
-              v(i,:)=undefg
+              z_count=z_count+1
+              vecz(i,j)=0.0
            end if
         end do
-     end if
-  else
-     do i=2,nr
-        call Anomaly_1d( vecz(i,:,1), v(i,:) )
      end do
   end if
 
-  v(1,:)=0.0
+  if(present(undef))then
+     if(present(undefg))then
+        call tangent_mean_anom_scal( x, y, xc, yc, vecz, r, theta, v,  &
+  &                                  undef=undef, undefg=undefg,  &
+  &                                  undefgc=trim(undefgc) )
+     else
+        call tangent_mean_anom_scal( x, y, xc, yc, vecz, r, theta, v,  &
+  &                                  undef=undef )
+     end if
+  else if(present(undefg))then
+     call tangent_mean_anom_scal( x, y, xc, yc, vecz, r, theta, v,  &
+  &                               undefg=undefg, undefgc=trim(undefgc) )
+  else
+     call tangent_mean_anom_scal( x, y, xc, yc, vecz, r, theta, v )
+  end if
 
 end subroutine tangent_mean_anom_vec
 
@@ -1432,6 +1199,163 @@ subroutine grad_wind_pres( r, coril, v, rho, r_ref, p_ref, pres )
 
 end subroutine
 
+!--------------------------------------------------------------
+!--------------------------------------------------------------
+
+subroutine DC_Braun( x, y, fg, pres, search_dis, var_dis, center, undef )
+! Braun (2002) ˡ濴ꤹ.
+  use Math_Const
+  use Max_Min
+  use Statistics
+  implicit none
+  real, intent(in) :: x(:)  ! x κɸ [m]
+  real, intent(in) :: y(:)  ! y κɸ [m]
+  integer, intent(in) :: fg(2)
+                         ! 濴 (̾ﵤκ)
+                         ! fg(1) = x ֹ, fg(2) = y ֹ
+  real, intent(in) :: pres(size(x),size(y))
+                         ! ٤Ǥε (ɽ̵Ǥ褤.)
+                         ! , ɽ̵ξ, ̹Ƥ.
+  real, intent(in) :: search_dis  ! ΰ (fg ΰ֤濴)
+                         ! 㤨, 100000.0 ʤ, fg 濴˽Ĳ 100 km 
+  real, intent(in) :: var_dis  ! 濴֤к׻Ⱦ [m]
+  integer, intent(inout) :: center(2)  ! ᤿濴γǿ
+  real, intent(in), optional :: undef  ! ̤ͤ, ̤.
+                             ! ܥ롼Ǥ, ̤ͤ, 
+                             ! γʻΤк׻˻Ѥʤ.
+
+  integer :: i, j, ix, jy, nx, ny, nr, nt, nxgmin, nxgmax, nygmin, nygmax
+  integer :: ompnum, tmp_o_num, is_dis
+  real :: dr, dtheta, xc, yc, undeff, tmpmin, tmp_anom, tmp_counter
+  real, allocatable, dimension(:) :: rad, theta
+  real, allocatable, dimension(:,:) :: anom_check
+  real, allocatable, dimension(:,:,:) :: apres
+
+!-- OpenMP ؿ
+!$ integer :: OMP_GET_THREAD_NUM, OMP_GET_MAX_THREADS
+
+  nx=size(x)
+  ny=size(y)
+
+  if(present(undef))then
+     undeff=undef
+  else
+     undeff=-999.0
+  end if
+
+!-- ʻϰϤ
+
+  call interpo_search_2d( x, y, x(fg(1))-0.5*search_dis,  &
+  &                             y(fg(2))-0.5*search_dis,  &
+  &                       nxgmin, nygmin, undeff=int(undeff) )
+
+  call interpo_search_2d( x, y, x(fg(1))+0.5*search_dis,  &
+  &                             y(fg(2))+0.5*search_dis,  &
+  &                       nxgmax, nygmax, undeff=int(undeff) )
+
+  if(nxgmin==int(undeff))then  ! ΰ賰Ȥξν
+     nxgmin=1
+  end if
+  if(nygmin==int(undeff))then  ! ΰ賰Ȥξν
+     nygmin=1
+  end if
+  if(nxgmax==int(undeff))then  ! ΰ賰Ȥξν
+     nxgmax=1
+  end if
+  if(nygmax==int(undeff))then  ! ΰ賰Ȥξν
+     nygmax=1
+  end if
+
+!  nxgmin=fg(1)-(search_dis-1)/2
+!  nxgmax=fg(1)+(search_dis-1)/2
+!  nygmin=fg(2)-(search_dis-1)/2
+!  nygmax=fg(2)+(search_dis-1)/2
+
+  allocate(anom_check(nx,ny))
+
+!-- openmp Ǥξդѥ
+!-- ʿѥΥޥβս openmp 󤷤,
+!-- apres  inout °ʤΤ, private °ꤷʤ
+!-- thread Ȥ apres ͤ񤭤Ƥޤ.
+!-- , threads number 򻲾Ȥ 3 ˤ,
+!-- thread Ȥ̤Ȥ褦ѹ.
+
+   ompnum=1
+!$   ompnum=OMP_GET_MAX_THREADS()  ! OpenMP ͭξϤͭ.
+
+  allocate(apres(nx,ny,ompnum))
+
+!-- ϤؤѴκݤˤ, var_dis Ǥ٤ x, y ʤ褦
+!-- ꤹ.
+
+  nr=int(var_dis/(x(2)-x(1)))+1
+  nt=int(2.0*pi*var_dis/(x(2)-x(1)))+1
+!  nt=4
+  dr=x(2)-x(1)
+  dtheta=2.0*pi/real(nt-1)
+
+  allocate(rad(nr))
+  allocate(theta(nt))
+
+  rad=(/((dr*real(i-1)),i=1,nr)/)
+  theta=(/((dtheta*real(i-1)),i=1,nt)/)
+
+!-- õʻˤĤ, ʿкȤ,
+!-- Ƴʻˤкιפ׻.
+
+  anom_check=undeff  ! õϰϳγʻˤϤ٤ undeff .
+                     ! õϰγʻҤˤϤȤǥ.
+
+  tmp_o_num=1  ! OpenMP ͭǤʤ, ͤ apres  3 ܤ
+
+!$omp parallel default(shared)
+!$omp do schedule(dynamic) private(i,j,tmp_anom,tmp_o_num,tmp_counter)
+
+  do j=nygmin,nygmax
+     do i=nxgmin,nxgmax
+
+        if(pres(i,j)/=undeff)then
+!$         tmp_o_num=OMP_GET_THREAD_NUM()+1  ! OpenMP ͭξ, ͤ apres  3 ܤ
+
+           apres(:,:,tmp_o_num)=0.0
+           tmp_counter=0
+
+           call tangent_mean_anom_scal_Cart( x, y, x(i), y(j), pres, rad, theta,  &
+  &                                          apres(:,:,tmp_o_num), undef=undeff,  &
+  &                                          undefg=undeff, undefgc='inc' )
+
+           tmp_anom=0.0
+
+           do jy=1,ny
+              do ix=1,nx
+                 if(apres(ix,jy,tmp_o_num)/=undeff)then
+                    tmp_counter=tmp_counter+1
+                    tmp_anom=tmp_anom+apres(ix,jy,tmp_o_num)*apres(ix,jy,tmp_o_num)
+                 end if
+              end do
+           end do
+
+           if(tmp_counter>0)then  ! ʿѤ undef ʤȤϹʤ.
+              anom_check(i,j)=tmp_anom
+           end if
+        end if
+     end do
+  end do
+
+!$omp end do
+!$omp end parallel
+
+!-- ׻кιͤΤ, ǾȤʤʻ.
+
+  call min_val_2d( anom_check, center(1), center(2), tmpmin, undef=undeff )
+
+  deallocate(rad)
+  deallocate(theta)
+  deallocate(anom_check)
+  deallocate(apres)
+
+end subroutine DC_Braun
+
 !--------------------------------------------------------------
 !  ʲ, private 롼
 !--------------------------------------------------------------
@@ -1591,7 +1515,7 @@ end function
 !
 ! ֤®˰٤ѴͿ, Ǥμžޤγѱư̤
 ! ׻뤳Ȥǽ.
-! ٥ȥγѷ׻롼 vec_prod Ѥ뤳ȤǶ˺ɸǤ׻ǽ.
+! ٥ȥγѷ׻롼 vec_prod_2d Ѥ뤳ȤǶ˺ɸǤ׻ǽ.
 !  use algebra
 !  implicit none
 !  real, intent(in) :: x(:)  ! x ΰֺɸ
@@ -1619,7 +1543,7 @@ end function
 !!$omp end parallel do
 !
 !  tmp=0.0
-!  call vec_prod(xxp,yyp,tmp,u,v,tmp,tmp,tmp,mome)
+!  call vec_prod_2d(xxp,yyp,tmp,u,v,tmp,tmp,tmp,mome)
 !
 !end subroutine Rangular_moment
 !
@@ -1634,7 +1558,7 @@ end function
 !! ֤®˰٤ѴͿ, Ǥμžޤγѱư̤
 !! ׻뤳Ȥǽ.
 !!
-!! ٥ȥγѷ׻롼 vec_prod Ѥ뤳ȤǶ˺ɸǤ׻ǽ.
+!! ٥ȥγѷ׻롼 vec_prod_2d Ѥ뤳ȤǶ˺ɸǤ׻ǽ.
 !  use algebra
 !  implicit none
 !  real, intent(in) :: x(:)  ! x ΰֺɸ
@@ -1663,7 +1587,7 @@ end function
 !!$omp end parallel do
 !
 !  tmp=0.0
-!  call vec_prod(xxp,yyp,tmp,u,v,tmp,tmp,tmp,mome)
+!  call vec_prod_2d(xxp,yyp,tmp,u,v,tmp,tmp,tmp,mome)
 !  call radius(xp,yp,0.0,x,y,tmp1,rp)
 !
 !!$omp parallel do shared(mome,f,rp) private(i,j)
