@@ -144,9 +144,32 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
 
 end subroutine rectangle_int
 
+!------------------------------------------
 !------------------------------------------
 
-subroutine abst(x,y,z,dis)  ! 3 ٥ȥͤ׻롼
+subroutine abst_2d(x,y,dis)
+  ! 2 ٥ȥͤ׻롼
+  implicit none
+  real, intent(in) :: x(:,:)  ! x Υ٥ȥʬ
+  real, intent(in) :: y(size(x,1),size(x,2))  ! y Υ٥ȥʬ
+  real, intent(inout) :: dis(size(x,1),size(x,2))  ! Ǥͥ٥ȥ
+  integer :: i, j, nx, ny
+
+  nx=size(x,1)
+  ny=size(x,2)
+
+  do j=1,ny
+     do i=1,nx
+        dis(i,j)=sqrt(x(i,j)**2+y(i,j)**2)
+     end do
+  end do
+
+end subroutine abst_2d
+
+!------------------------------------------
+!------------------------------------------
+
+subroutine abst_3d(x,y,z,dis)  ! 3 ٥ȥͤ׻롼
   ! Ĵ뤳Ȥˤ, 2 Ǥη׻ǽ.
   implicit none
   real, intent(in) :: x(:,:,:)  ! x Υ٥ȥʬ
@@ -171,8 +194,9 @@ subroutine abst(x,y,z,dis)  ! 3 ٥ȥͤ׻롼
 !$omp end do
 !$omp end parallel
 
-end subroutine abst
+end subroutine abst_3d
 
+!------------------------------------------
 !------------------------------------------
 
 subroutine calc_radius(xp,yp,zp,x,y,z,rad)
@@ -210,7 +234,50 @@ end subroutine calc_radius
 !------------------------------------------
 !------------------------------------------
 
-subroutine dot_prod(x,y,z,u,v,w,dot,undeff)
+subroutine dot_prod_2d(x,y,u,v,dot,undeff)
+  ! 2٥ȥѷ׻롼
+  ! dot_prod  2 
+  implicit none
+  real, intent(in) :: x(:,:)  ! x Υ٥ȥʬ
+  real, intent(in) :: y(size(x,1),size(x,2))  ! y Υ٥ȥʬ
+  real, intent(in) :: u(size(x,1),size(x,2))  ! x Υ٥ȥʬ
+  real, intent(in) :: v(size(x,1),size(x,2))  ! y Υ٥ȥʬ
+  real, intent(inout) :: dot(size(x,1),size(x,2))  ! 
+  real, intent(in), optional :: undeff        ! ̤
+  integer :: i, j, nx, ny
+
+  nx=size(x,1)
+  ny=size(x,2)
+
+  if(present(undeff))then
+
+     do j=1,ny
+        do i=1,nx
+           if(x(i,j)==undeff.or.u(i,j)==undeff.or.  &
+  &           y(i,j)==undeff.or.v(i,j)==undeff)then
+              dot(i,j)=undeff
+           else
+              dot(i,j)=x(i,j)*u(i,j)+y(i,j)*v(i,j)
+           end if
+        end do
+     end do
+
+  else
+
+     do j=1,ny
+        do i=1,nx
+           dot(i,j)=x(i,j)*u(i,j)+y(i,j)*v(i,j)
+        end do
+     end do
+
+  end if
+
+end subroutine dot_prod_2d
+
+!------------------------------------------
+!------------------------------------------
+
+subroutine dot_prod_3d(x,y,z,u,v,w,dot,undeff)
   ! 2٥ȥѷ׻롼
   ! פ, 2 ǤѤ׻뤳Ȥǽ
   implicit none
@@ -262,12 +329,58 @@ subroutine dot_prod(x,y,z,u,v,w,dot,undeff)
 !$omp end parallel
   end if
 
-end subroutine dot_prod
+end subroutine dot_prod_3d
 
+!------------------------------------------
+!------------------------------------------
 
-subroutine vec_prod(x,y,z,u,v,w,vecx,vecy,vecz,undeff)
+subroutine vec_prod_2d(x,y,u,v,vecz,undeff)
+  ! 2٥ȥγѷ׻롼
+  ! vec_prod  2 
+  ! ѤʤΤ, 2 Ƿ׻, Ѥ 1 ʬʤΤ,
+  ! intent(inout) ΰ 1 ĤʤȤ.
+  implicit none
+  real, intent(in) :: x(:,:)  ! x Υ٥ȥʬ
+  real, intent(in) :: y(size(x,1),size(x,2))  ! y Υ٥ȥʬ
+  real, intent(in) :: u(size(x,1),size(x,2))  ! x Υ٥ȥʬ
+  real, intent(in) :: v(size(x,1),size(x,2))  ! y Υ٥ȥʬ
+  real, intent(inout) :: vecz(size(x,1),size(x,2))  ! Ѥľʬ
+  real, intent(in), optional :: undeff
+  integer :: i, j, nx, ny
+
+  nx=size(x,1)
+  ny=size(x,2)
+
+  if(present(undeff))then
+
+        do j=1,ny
+           do i=1,nx
+              if(x(i,j)==undeff.or.u(i,j)==undeff.or.  &
+  &              y(i,j)==undeff.or.v(i,j)==undeff)then
+                 vecz(i,j)=undeff
+              else
+                 vecz(i,j)=x(i,j)*v(i,j)-y(i,j)*u(i,j)
+              end if
+           end do
+        end do
+
+  else
+
+        do j=1,ny
+           do i=1,nx
+              vecz(i,j)=x(i,j)*v(i,j)-y(i,j)*u(i,j)
+           end do
+        end do
+
+  end if
+
+end subroutine vec_prod_2d
+
+!-----------------------------------------
+!-----------------------------------------
+
+subroutine vec_prod_3d(x,y,z,u,v,w,vecx,vecy,vecz,undeff)
   ! 2٥ȥγѷ׻롼
-  ! ǿפ뤳Ȥ 2 Ѥ׻ǽ
   implicit none
   real, intent(in) :: x(:,:,:)  ! x Υ٥ȥʬ
   real, intent(in) :: y(size(x,1),size(x,2),size(x,3))  ! y Υ٥ȥʬ
@@ -326,7 +439,7 @@ subroutine vec_prod(x,y,z,u,v,w,vecx,vecy,vecz,undeff)
 
   end if
 
-end subroutine vec_prod
+end subroutine vec_prod_3d
 
 !-----------------------------------------
 !-----------------------------------------
