@@ -8,6 +8,9 @@ module Basis  ! Ūʷ׻ؿ¹Ԥ⥸塼
      integer :: year_d  ! 
      integer :: month_d  ! 
      integer :: day_d  ! 
+     integer :: hour_d  ! 
+     integer :: min_d  ! ʬ
+     integer :: sec_d  ! 
   end type dtime
 
 contains
@@ -215,7 +218,7 @@ integer function counter_day( stime, etime )
   integer :: nsy, nsm, nsd, ney, nem, ned
 
   nt=etime%year_d-stime%year_d+1
-  nm=etime%month_d-stime%month_d+1
+  nm=etime%month_d-stime%month_d
   days=0
   nsy=stime%year_d
   nsm=stime%month_d
@@ -224,9 +227,10 @@ integer function counter_day( stime, etime )
   nem=etime%month_d
   ned=etime%day_d
 
-!-- ǯʬ, ǯǾʬ.
-  if(nt>2)then
-     do i=2,nt-1
+!-- ʣǯˤޤ, ǯǾʬ.
+!-- ǯ 1 ǯʬȤ(Ȥǰ).
+  if(nt>1)then
+     do i=1,nt-1
         year_tmp=nsy+i-1
         if(mod(year_tmp,4)==0)then
            year_fact=366
@@ -237,21 +241,13 @@ integer function counter_day( stime, etime )
      end do
   end if
 
-!-- νԤäƤΤ, ­碌ΤϳǯȽλǯΤ.
+!-- νԤäƤΤ, ȤϽλǯ­,
+!-- ǯǳޤǤҤΤ.
 !-- ޤ, 1 ޤޤ¸ߤ­碌.
 !-- ,  12 , λ 1 ξ, ­­Τ,
 !-- ǤϥȤʤ.
   if(nt>1)then  ! ǯޤ
-     if(nsm<12)then  !  12 ǤϤʤ.
-        do i=nsm+1,12
-           if(i==2.and.mod(nsy,4)==0)then
-              days=days+month(13)
-           else
-              days=days+month(i)
-           end if
-        end do
-     end if
-
+     ! λ­碌
      if(nem>1)then  ! λ 1 ǤϤʤ.
         do i=1,nem-1
            if(i==2.and.mod(ney,4)==0)then
@@ -261,9 +257,20 @@ integer function counter_day( stime, etime )
            end if
         end do
      end if
+
+     if(nsm>1)then  !  1 ǤϤʤ.
+        do i=1,nsm-1
+           if(i==2.and.mod(nsy,4)==0)then
+              days=days-month(13)
+           else
+              days=days-month(i)
+           end if
+        end do
+     end if
+
   else
-     if(nm>2)then  ! ǯޤ餺, ޤ, Ĵ֤ 1 ʬ
-                   ! ޤޤäƤ.
+
+     if(nm/=1)then  ! ǯޤ餺, ޤ.
         do i=nsm+1,nem-1
            if(i==2.and.mod(ney,4)==0)then
               days=days+month(13)
@@ -275,21 +282,21 @@ integer function counter_day( stime, etime )
   end if
 
 !-- , ϷȽλ­碌.
-  if(nt/=1.or.nm/=1)then  ! ǯ⤫֤äƤʤ
-     if(mod(nsy,4)==0.and.nsm==2)then
-        days=days+month(13)-nsd
-     else
-        days=days+month(nsm)-nsd
-     end if
+!  if(nt/=1.or.nm/=1)then  ! ǯ⤫֤äƤʤ
+!     if(mod(nsy,4)==0.and.nsm==2)then
+!        days=days+month(13)-nsd
+!     else
+!        days=days+month(nsm)-nsd
+!     end if
 
-     if(mod(ney,4)==0.and.nem==2)then
-        days=days+ned
-     else
-        days=days+ned
-     end if
-  else
-     days=days+ned-nsd
-  end if
+!     if(mod(ney,4)==0.and.nem==2)then
+!        days=days+ned
+!     else
+!        days=days+ned
+!     end if
+!  end if
+
+  days=days+ned-nsd
 
   days=days+1
   counter_day=days
@@ -297,4 +304,125 @@ integer function counter_day( stime, etime )
   return
 end function
 
+!-----------------------------------
+!-----------------------------------
+
+integer function counter_sec( stime, etime )
+! 齪λޤǤÿ򥫥Ȥ.
+  implicit none
+  type(dtime), intent(in) :: stime  ! 
+  type(dtime), intent(in) :: etime  ! λ
+  integer :: nday, tmp_sec
+  integer :: nsh, nsm, nss, neh, nem, nes
+
+  tmp_sec=0
+
+!-- ޤ, 򥫥Ȥ.
+
+  nday=counter_day( stime, etime )
+
+!-- 򸵤, ʬä򥫥.
+! counter_day  1 ׻Ƥ뤫
+! ƱǤ counter_day = 1, 24 ְޤ counter_day = 2.
+
+  if(nday>2)then
+     tmp_sec=(nday-2)*86400
+  end if
+
+!-- stime  etime ޤǤ hour, minite, sec ׻, ȡä֤.
+
+  nsh=stime%hour_d
+  nsm=stime%min_d
+  nss=stime%sec_d
+  neh=etime%hour_d
+  nem=etime%min_d
+  nes=etime%sec_d
+
+  if(nday==1)then  !  1 ¸ߤ, hour Τ߷׻.
+     tmp_sec=tmp_sec+(neh-nsh)*3600+(nem-nsm)*60+(nes-nss)
+  else  ! ޤäƤ, stime  86400 s ǥȤƤ
+        ! nsh, nsm, nss Фä׻Ǥ.
+     tmp_sec=tmp_sec+86400  ! stime  1 ʬ
+     tmp_sec=tmp_sec+neh*3600+nem*60+nem  ! etime ä­
+     tmp_sec=tmp_sec-nsh*3600-nsm*60-nsm  ! stime ä
+  end if
+
+  counter_sec=tmp_sec
+
+  return
+end function
+
+!-----------------------------------
+!-----------------------------------
+
+subroutine time_zone_convert( factor, itime, ctime )
+! ॾѴԤ롼
+! 㤨, JST  UTC Ѵ, factor = -9 ȤФ褤.
+  implicit none
+  integer, intent(in) :: factor        ! Ѳ뤫.
+  type(dtime), intent(in) :: itime     ! Ѵλ
+  type(dtime), intent(inout) :: ctime  ! Ѵλ
+  integer :: iyear, imonth, iday, ihour
+  integer, parameter, dimension(13) :: month=(/31,28,31,30,31,30,  &
+  &                                            31,31,30,31,30,31,29/)
+
+  iyear=itime%year_d
+  imonth=itime%month_d
+  iday=itime%day_d
+  ihour=itime%hour_d
+
+  ihour=ihour+factor
+
+  if(ihour<1)then
+     do while(ihour<1)
+        iday=iday-1
+        ihour=ihour+24
+     end do
+  else if(ihour>=24)then
+     do while(ihour>=24)
+        iday=iday+1
+        ihour=ihour-24
+     end do
+  end if
+
+  if(iday<1)then
+     do while(iday<1)
+        if(mod(iyear,4)==0.and.imonth==2)then
+           iday=iday+month(13)
+        else
+           iday=iday+month(imonth)
+        end if
+
+        imonth=imonth-1
+
+        if(imonth<=0)then
+           iyear=iyear-1
+           imonth=12
+        end if
+     end do
+  else if(iday>month(imonth))then
+     do while(iday>month(imonth))
+        if(mod(iyear,4)==0.and.imonth==2)then
+           iday=iday-month(13)
+        else
+           iday=iday-month(imonth)
+        end if
+        imonth=imonth+1
+        if(imonth>12)then
+           iyear=iyear+1
+           imonth=1
+        end if
+     end do
+  end if
+
+  ctime%year_d=iyear
+  ctime%month_d=imonth
+  ctime%day_d=iday
+  ctime%hour_d=ihour
+
+end subroutine
+
+!-----------------------------------
+!-----------------------------------
+
 end module
